@@ -89,7 +89,6 @@ from glue.ligolw import ligolw
 from glue.ligolw.utils import process as ligolw_process
 from glue.ligolw.utils import search_summary as ligolw_search_summary
 from glue.ligolw import table as ligolw_table
-from pylal import ligolw_thinca
 from glue.ligolw import utils as ligolw_utils
 from glue.ligolw import lsctables
 
@@ -172,13 +171,13 @@ except ValueError:
     out_xmldoc.childNodes[0].appendChild(time_slide_table)
     time_slide_table.sync_next_id()
 time_slide_id = time_slide_table.get_time_slide_id(
-    dict((ifo, 0) for ifo in opts.detector), create_new=process)
+    {ifo: 0 for ifo in opts.detector}, create_new=process)
 
 # Create a CoincDef table and record a CoincDef row for
 # sngl_inspiral <-> sngl_inspiral coincidences.
 coinc_def_table = lsctables.New(lsctables.CoincDefTable)
 out_xmldoc.childNodes[0].appendChild(coinc_def_table)
-coinc_def = ligolw_thinca.InspiralCoincDef
+coinc_def = ligolw_bayestar.InspiralCoincDef
 coinc_def_id = coinc_def_table.get_next_id()
 coinc_def.coinc_def_id = coinc_def_id
 coinc_def_table.append(coinc_def)
@@ -200,9 +199,9 @@ progress.update(-1, 'reading ' + opts.reference_psd.name)
 xmldoc, _ = ligolw_utils.load_fileobj(
     opts.reference_psd, contenthandler=lal.series.PSDContentHandler)
 psds = lal.series.read_psd_xmldoc(xmldoc)
-psds = dict(
-    (key, timing.InterpolatedPSD(filter.abscissa(psd), psd.data.data))
-    for key, psd in psds.items() if psd is not None)
+psds = {
+    key: timing.InterpolatedPSD(filter.abscissa(psd), psd.data.data)
+    for key, psd in psds.items() if psd is not None}
 
 # Detector noise PSD model
 class psdfunc(object):
@@ -427,8 +426,9 @@ progress.update(-1, 'writing ' + opts.output.name)
 ligolw_process.set_process_end_time(process)
 
 # Write output file.
-ligolw_utils.write_fileobj(out_xmldoc, opts.output,
-    gz=(os.path.splitext(opts.output.name)[-1]==".gz"))
+with ligolw_utils.SignalsTrap():
+  ligolw_utils.write_fileobj(out_xmldoc, opts.output,
+      gz=(os.path.splitext(opts.output.name)[-1]==".gz"))
 
 
 if handler.interrupted:
