@@ -61,13 +61,10 @@ static struct itimerval checkpoint_timer;
 /** Utility functions for the resume functionality */
 /** Write the current state to a checkpoint file the given filename */
 /** Read the given filename to populate a given LALInferenceRunState and NSintegralState */
-//#ifdef HAVE_HDF5
-#if 0
+#ifdef HAVE_HDF5
 static int _saveNSintegralStateH5(LALH5File *group, NSintegralState *s);
 static int _saveNSintegralStateH5(LALH5File *group, NSintegralState *s)
 {
-  UINT4 N=s->size;
-  XLALH5FileAddScalarAttribute(group, "number_live_points", &(N), LAL_U4_TYPE_CODE );
   XLALH5FileAddScalarAttribute(group, "iteration", &(s->iteration), LAL_U4_TYPE_CODE );
   XLALH5FileWriteREAL8Vector(group, "logZarray", s->logZarray);
   XLALH5FileWriteREAL8Vector(group, "oldZarray", s->oldZarray);
@@ -81,7 +78,6 @@ static int _saveNSintegralStateH5(LALH5File *group, NSintegralState *s)
 static int _loadNSintegralStateH5(LALH5File *group, NSintegralState *s);
 static int _loadNSintegralStateH5(LALH5File *group, NSintegralState *s)
 {
-  XLALH5FileQueryScalarAttributeValue(&(s->size), group, "number_live_points");
   XLALH5FileQueryScalarAttributeValue(&(s->iteration), group, "iteration");
   s->logZarray = XLALH5FileReadREAL8Vector(group, "logZarray");
   s->oldZarray = XLALH5FileReadREAL8Vector(group, "oldZarray");
@@ -89,6 +85,7 @@ static int _loadNSintegralStateH5(LALH5File *group, NSintegralState *s)
   s->logwarray = XLALH5FileReadREAL8Vector(group, "logwarray");
   s->logtarray = XLALH5FileReadREAL8Vector(group, "logtarray");
   s->logt2array = XLALH5FileReadREAL8Vector(group, "logt2array");
+  s->size = s->logZarray->length;
   return(0);
 }
 
@@ -109,14 +106,14 @@ static int WriteNSCheckPointH5(char *filename, LALInferenceRunState *runState, N
   if(!group) XLAL_ERROR(XLAL_EFAILED,"Unable to read group lalinferencenest_checkpoint\n");
   int retcode = _saveNSintegralStateH5(group,s);
   if(retcode) XLAL_ERROR(XLAL_EFAILED,"Unable to save integral state\n");
-  LALInferenceH5VariablesArray2Group(group, runState->livePoints, Nlive, "live_points");
+  LALInferenceH5VariablesArrayToDataset(group, runState->livePoints, Nlive, "live_points");
   INT4 N_output_array=0;
   if(LALInferenceCheckVariable(runState->algorithmParams,"N_outputarray")) N_output_array=LALInferenceGetINT4Variable(runState->algorithmParams,"N_outputarray");
   if(N_output_array>0)
   {
     LALInferenceVariables **output_array=NULL;
     output_array=*(LALInferenceVariables ***)LALInferenceGetVariable(runState->algorithmParams,"outputarray");
-    LALInferenceH5VariablesArray2Group(group, output_array, N_output_array, "past_chain");
+    LALInferenceH5VariablesArrayToDataset(group, output_array, N_output_array, "past_chain");
     XLALH5FileAddScalarAttribute(group, "N_outputarray", &N_output_array, LAL_I4_TYPE_CODE);
   }
   
@@ -129,6 +126,7 @@ static int ReadNSCheckPointH5(char *filename, LALInferenceRunState *runState, NS
 {
   int retcode;
   LALH5File *h5file;
+  UINT4 Nlive;
   if( access( filename, F_OK ) == -1 ) return(1);
   XLAL_TRY(h5file = XLALH5FileOpen(filename,"r"),retcode);
   if(retcode!=XLAL_SUCCESS) return(retcode);
@@ -150,18 +148,18 @@ static int ReadNSCheckPointH5(char *filename, LALInferenceRunState *runState, NS
     XLALH5FileClose(h5file);
     return 1;
   }
-  LALH5File *liveGroup = XLALH5GroupOpen(group,"live_points");
-  retcode = LALInferenceH5GroupToVariablesArray(liveGroup , &(runState->livePoints), &(s->size));
-  printf("restored %i live points\n",s->size);
-  XLALH5FileClose(liveGroup);
+  LALH5Dataset *liveGroup = XLALH5DatasetRead(group,"live_points");
+  retcode = LALInferenceH5DatasetToVariablesArray(liveGroup , &(runState->livePoints), &Nlive );
+  printf("restored %i live points\n",Nlive);
+  XLALH5DatasetFree(liveGroup);
   if(N_outputarray>0)
   {
     printf("restoring %i past iterations\n",N_outputarray);
-    LALH5File *outputGroup = XLALH5GroupOpen(group, "past_chain");
-    retcode |= LALInferenceH5GroupToVariablesArray(outputGroup, &outputarray, &N_outputarray);
+    LALH5Dataset *outputGroup = XLALH5DatasetRead(group, "past_chain");
+    retcode |= LALInferenceH5DatasetToVariablesArray(outputGroup, &outputarray, &N_outputarray);
     LALInferenceAddVariable(runState->algorithmParams,"N_outputarray",&N_outputarray,LALINFERENCE_INT4_t,LALINFERENCE_PARAM_OUTPUT);
     LALInferenceAddVariable(runState->algorithmParams,"outputarray",&outputarray,LALINFERENCE_void_ptr_t,LALINFERENCE_PARAM_OUTPUT);
-    XLALH5FileClose(outputGroup);
+    XLALH5DatasetFree(outputGroup);
   }
   XLALH5FileClose(group);
   XLALH5FileClose(h5file);
@@ -212,6 +210,7 @@ static int _loadNSintegralState(FILE *fp, NSintegralState *s)
 static int WriteNSCheckPoint(CHAR *filename, LALInferenceRunState *runState, NSintegralState *s)
 {
   FILE *progfile=fopen(filename,"w");
+  int errnum;
   if(!progfile)
   {
     fprintf(stderr,"Unable to save resume file %s!\n",filename);
@@ -228,7 +227,13 @@ static int WriteNSCheckPoint(CHAR *filename, LALInferenceRunState *runState, NSi
       fclose(progfile);
       return 1;
     }
-    LALInferenceWriteVariablesArrayBinary(progfile,runState->livePoints, Nlive);
+    XLAL_TRY(LALInferenceWriteVariablesArrayBinary(progfile,runState->livePoints, Nlive),errnum);
+	if(errnum!=XLAL_SUCCESS)
+	{
+      fprintf(stderr,"Unable to write live points - will not be able to resume!\n");
+      fclose(progfile);
+      return 1;
+	}
     INT4 N_output_array=0;
     if(LALInferenceCheckVariable(runState->algorithmParams,"N_outputarray")) N_output_array=LALInferenceGetINT4Variable(runState->algorithmParams,"N_outputarray");
     fwrite(&N_output_array,sizeof(INT4),1,progfile);
@@ -236,7 +241,13 @@ static int WriteNSCheckPoint(CHAR *filename, LALInferenceRunState *runState, NSi
     {
       LALInferenceVariables **output_array=NULL;
       output_array=*(LALInferenceVariables ***)LALInferenceGetVariable(runState->algorithmParams,"outputarray");
-      LALInferenceWriteVariablesArrayBinary(progfile,output_array, N_output_array);
+      XLAL_TRY(LALInferenceWriteVariablesArrayBinary(progfile,output_array, N_output_array),errnum);
+	  if(errnum!=XLAL_SUCCESS)
+	  {
+		      fprintf(stderr,"Unable to write past chain - will not be able to resume!\n");
+		      fclose(progfile);
+		      return 1;	
+	  }
       fprintf(stderr,"Resume --> wrote %d past chain samples\n\n",N_output_array);
     }
     fclose(progfile);
@@ -246,6 +257,7 @@ static int WriteNSCheckPoint(CHAR *filename, LALInferenceRunState *runState, NSi
 
 static int ReadNSCheckPoint(CHAR *filename, LALInferenceRunState *runState, NSintegralState *s)
 {
+  int errnum;
   FILE *progfile=fopen(filename,"r");
   if(!progfile)
   {
@@ -262,14 +274,26 @@ static int ReadNSCheckPoint(CHAR *filename, LALInferenceRunState *runState, NSin
       return 1;
     }
     //if(retcode) return 1;
-    LALInferenceReadVariablesArrayBinary(progfile,runState->livePoints,Nlive);
+    XLAL_TRY(LALInferenceReadVariablesArrayBinary(progfile,runState->livePoints,Nlive),errnum);
+	if(errnum!=XLAL_SUCCESS)
+    {
+		fprintf(stderr,"Unable to read live points from resume file!\n");
+		fclose(progfile);
+		return(1);
+	}
     INT4 N_output_array;
     fread(&N_output_array,sizeof(INT4),1,progfile);
     LALInferenceVariables **output_array=NULL;
     if(N_output_array!=0){
       output_array=XLALCalloc(N_output_array,sizeof(LALInferenceVariables *));
       fprintf(stderr,"Resume --> read %d past chain samples\n",N_output_array);
-      LALInferenceReadVariablesArrayBinary(progfile,output_array,N_output_array);
+      XLAL_TRY(LALInferenceReadVariablesArrayBinary(progfile,output_array,N_output_array),errnum);
+	  if(errnum!=XLAL_SUCCESS)
+	  {
+		fprintf(stderr,"Unable to read past chain from resume file!\n");
+		fclose(progfile);
+		return(1);
+	  }
       if(LALInferenceCheckVariable(runState->algorithmParams,"N_outputarray")) LALInferenceRemoveVariable(runState->algorithmParams,"N_outputarray");
       if(LALInferenceCheckVariable(runState->algorithmParams,"outputarray")) LALInferenceRemoveVariable(runState->algorithmParams,"outputarray");
       LALInferenceAddVariable(runState->algorithmParams,"N_outputarray",&N_output_array,LALINFERENCE_INT4_t,LALINFERENCE_PARAM_OUTPUT);
@@ -319,6 +343,20 @@ static REAL8 mean(REAL8 *array,int N);
 /** Calculate covariance matrix from a collection of live points */
 static void LALInferenceNScalcCVM(gsl_matrix **cvm, LALInferenceVariables **Live, UINT4 Nlive);
 
+/* log( exp(a) - exp(b) ) */
+static double logsubexp(double a, double b);
+static double logsubexp(double a, double b)
+{
+		if(b>a)
+		{
+				fprintf(stderr,"Cannot take log of negative number %lf - %lf = %lf !\n",exp(a),exp(b),exp(a)-exp(b));
+				return -INFINITY;
+		}
+		else
+		{
+				return a + log1p(-exp(b-a));
+		}
+}
 
 static void SetupEigenProposals(LALInferenceRunState *runState);
 
@@ -326,21 +364,21 @@ static void SetupEigenProposals(LALInferenceRunState *runState);
  * Update the internal state of the integrator after receiving the lowest logL
  * value logL
  */
-static REAL8 incrementEvidenceSamples(LALInferenceRunState *runState, REAL8 logL, NSintegralState *s);
-static REAL8 incrementEvidenceSamples(LALInferenceRunState *runState, REAL8 logL, NSintegralState *s)
+static REAL8 incrementEvidenceSamples(gsl_rng *GSLrandom, UINT4 Nlive, REAL8 logL, NSintegralState *s);
+static REAL8 incrementEvidenceSamples(gsl_rng *GSLrandom, UINT4 Nlive, REAL8 logL, NSintegralState *s)
 {
   REAL8 Wtarray[s->size];
-  UINT4 Nlive=*(UINT4 *)LALInferenceGetVariable(runState->algorithmParams,"Nlive");
+  //UINT4 Nlive=*(UINT4 *)LALInferenceGetVariable(runState->algorithmParams,"Nlive");
   /* Update evidence array */
   for(UINT4 j=0;j<s->size;j++){
-    Wtarray[j]=s->logwarray->data[j]+logL+logaddexp(0,s->logt2array->data[j]*s->logtarray->data[j])-log(2.0);
+    Wtarray[j]=s->logwarray->data[j]+logL+logsubexp(0,s->logt2array->data[j] + s->logtarray->data[j])-log(2.0);
     s->logZarray->data[j]=logaddexp(s->logZarray->data[j],Wtarray[j]);
     s->Harray->data[j]= exp(Wtarray[j]-s->logZarray->data[j])*logL
     + exp(s->oldZarray->data[j]-s->logZarray->data[j])*(s->Harray->data[j]+s->oldZarray->data[j])-s->logZarray->data[j];
     REAL8 tmp=s->logtarray->data[j];
     s->logtarray->data[j]=s->logt2array->data[j];
     s->logt2array->data[j]=tmp;
-    s->logtarray->data[j]=LALInferenceNSSample_logt(Nlive,runState->GSLrandom);
+    s->logtarray->data[j]=LALInferenceNSSample_logt(Nlive,GSLrandom);
     s->logwarray->data[j]+=s->logtarray->data[j];
   }
   s->iteration++;
@@ -381,12 +419,12 @@ static NSintegralState *initNSintegralState(UINT4 Nruns, UINT4 Nlive)
   s->logwarray = XLALCreateREAL8Vector(Nruns);
   s->logtarray=XLALCreateREAL8Vector(Nruns);
   s->logt2array=XLALCreateREAL8Vector(Nruns);
-  REAL8 logw=log(1.0-exp(-1.0/Nlive));
+  REAL8 logw=0;
 
   if(s->logZarray==NULL || s->Harray==NULL || s->oldZarray==NULL || s->logwarray==NULL)
   {fprintf(stderr,"Unable to allocate RAM\n"); exit(-1);}
-  for(UINT4 i=0;i<Nruns;i++)  {s->logwarray->data[i]=logw; s->logZarray->data[i]=-DBL_MAX;
-    s->oldZarray->data[i]=-DBL_MAX; s->Harray->data[i]=0.0;s->logtarray->data[i]=-1.0/Nlive;
+  for(UINT4 i=0;i<Nruns;i++)  {s->logwarray->data[i]=logw; s->logZarray->data[i]=-INFINITY;
+    s->oldZarray->data[i]=-INFINITY; s->Harray->data[i]=0.0;s->logtarray->data[i]=-1.0/Nlive;
     s->logt2array->data[i]=-1.0/Nlive;
   }
   return s;
@@ -587,7 +625,6 @@ void LALInferenceNestedSamplingAlgorithmInit(LALInferenceRunState *runState)
     --- Nested Sampling Algorithm Parameters -----\n\
     ----------------------------------------------\n\
     --Nlive N                        Number of live points to use\n\
-    (--outhdf <filename.h5>)         HDF5 file output path\n\
     (--Nmcmc M)                      Over-ride auto chain length determination and use <M> MCMC samples\n\
     (--maxmcmc M)                    Use at most M MCMC points when autodetermining the chain (5000)\n\
     (--Nmcmcinitial M)               Use M MCMC points when initially resampling from the prior\n\
@@ -732,15 +769,15 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
   UINT4 iter=0,i,j,minpos;
   /* Single thread here */
   LALInferenceThreadState *threadState = runState->threads[0];
-
+  UINT4 HDFOUTPUT=1;
   UINT4 Nlive=*(UINT4 *)LALInferenceGetVariable(runState->algorithmParams,"Nlive");
   UINT4 Nruns=100;
   REAL8 *logZarray,*oldZarray,*Harray,*logwarray,*logtarray;
   REAL8 TOLERANCE=0.1;
-  REAL8 logZ,logZnew,logLmin,logLmax=-DBL_MAX,logLtmp,logw,H,logZnoise,dZ=0;
+  REAL8 logZ,logZnew,logLmin,logLmax=-INFINITY,logLtmp,logw,H,logZnoise,dZ=0;
   LALInferenceVariables *temp;
   FILE *fpout=NULL;
-  REAL8 dblmax=-DBL_MAX;
+  REAL8 neginfty=-INFINITY;
   REAL8 zero=0.0;
   REAL8 *logLikelihoods=NULL;
   UINT4 verbose=0;
@@ -785,7 +822,7 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
 
   /* Check that necessary parameters are created */
   if(!LALInferenceCheckVariable(runState->algorithmParams,"logLmin"))
-    LALInferenceAddVariable(runState->algorithmParams,"logLmin",&dblmax,LALINFERENCE_REAL8_t,LALINFERENCE_PARAM_OUTPUT);
+    LALInferenceAddVariable(runState->algorithmParams,"logLmin",&neginfty,LALINFERENCE_REAL8_t,LALINFERENCE_PARAM_OUTPUT);
 
   if(!LALInferenceCheckVariable(runState->algorithmParams,"accept_rate"))
     LALInferenceAddVariable(runState->algorithmParams,"accept_rate",&zero,LALINFERENCE_REAL8_t,LALINFERENCE_PARAM_OUTPUT);
@@ -798,23 +835,29 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
   ppt = NULL;
   ppt=LALInferenceGetProcParamVal(runState->commandLine,"--outfile");
   if(!ppt){
-    fprintf(stderr,"Must specify --outfile <filename.dat>\n");
-    exit(1);
+      fprintf(stderr,"Must specify --outfile <filename.hdf5>\n");
+      exit(1);
   }
   char *outfile=ppt->value;
-
+  /* Check if the output file has hdf5 extension */
+  if(strstr(outfile,".h5") || strstr(outfile,".hdf")) HDFOUTPUT=1;
+  else HDFOUTPUT=0;
+  
+#ifndef HAVE_HDF5
+  if(HDFOUTPUT)
+  {
+      fprintf(stderr,"Error: LALSuite was compiled without HDF5 support. Unable to write HDF5 output files\n");
+      exit(1);
+  }
+#endif
+  
+  double logvolume=0.0;
   if ( LALInferenceCheckVariable( runState->livePoints[0], "chirpmass" ) ){
     /* If a cbc run, calculate the mass-distance volume and store it to file*/ 
     /* Do it before algorithm starts so that we can kill the run and still get this */
-    double volume=LALInferenceMassDistancePriorVolume(runState);
-    char priorfile[FILENAME_MAX];
-    sprintf(priorfile,"%s_prior_weight.txt",outfile);
-    fpout=fopen(priorfile,"w");
-    fprintf(fpout,"%10.10e\n",volume);
-    fclose(fpout);
+    logvolume=log(LALInferenceMassDistancePriorVolume(runState));
   }
 
-
   if(LALInferenceGetProcParamVal(runState->commandLine,"--progress"))
     displayprogress=1;
 
@@ -846,11 +889,9 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
   /* Check for an interrupted run */
   char resumefilename[FILENAME_MAX];
   sprintf(resumefilename,"%s_resume",outfile);
-  char outfilebackup[FILENAME_MAX];
-  sprintf(outfilebackup,"%s.bak",outfile);
   int retcode=1;
   if(LALInferenceGetProcParamVal(runState->commandLine,"--resume")){
-#if 0
+#ifdef HAVE_HDF5
       retcode=ReadNSCheckPointH5(resumefilename,runState,s);
 #else
       retcode=ReadNSCheckPoint(resumefilename,runState,s);
@@ -858,13 +899,6 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
       if(retcode==0){
           for(i=0;i<Nlive;i++) logLikelihoods[i]=*(REAL8 *)LALInferenceGetVariable(runState->livePoints[i],"logL");
           iter=s->iteration;
-          /* back up the old file and get ready to append to it from the right place */
-          char commandline[1024];
-          sprintf(commandline,"cp %s %s",outfile,outfilebackup);
-          system(commandline);
-          sprintf(commandline,"head -n %i %s > %s",iter+1,outfilebackup,outfile);
-          system(commandline);
-          fpout=fopen(outfile,"a");
       }
       /* Install a periodic alarm that will trigger a checkpoint */
       int sigretcode=0;
@@ -876,7 +910,7 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
       /* Condor sends SIGUSR2 to checkpoint and continue */
       sigretcode=sigaction(SIGUSR2,&sa,NULL);
       if(sigretcode!=0) fprintf(stderr,"WARNING: Cannot establish checkpoint on SIGUSR2.\n");
-      checkpoint_timer.it_interval.tv_sec=4*3600; /* Default timer 4 hours */
+      checkpoint_timer.it_interval.tv_sec=30*60; /* Default timer 30 mins */
       checkpoint_timer.it_interval.tv_usec=0;
       checkpoint_timer.it_value=checkpoint_timer.it_interval;
       setitimer(ITIMER_VIRTUAL,&checkpoint_timer,NULL);
@@ -896,23 +930,19 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
     if(LALInferenceGetProcParamVal(runState->commandLine,"--resume"))
         fprintf(stdout,"Unable to open resume file %s. Starting anew.\n",resumefilename);
     /* Sprinkle points */
-    LALInferenceSetVariable(runState->algorithmParams,"logLmin",&dblmax);
+    LALInferenceSetVariable(runState->algorithmParams,"logLmin",&neginfty);
+    int sprinklewarning=0;
     for(i=0;i<Nlive;i++) {
-	threadState->currentParams=runState->livePoints[i];
-	do
-    {
-        runState->evolve(runState);
-    	logLikelihoods[i]=runState->likelihood(runState->livePoints[i],runState->data,threadState->model);
-    }while(isnan(logLikelihoods[i]));
-	if(XLALPrintProgressBar((double)i/(double)Nlive)) fprintf(stderr,"\n");
+	    threadState->currentParams=runState->livePoints[i];
+	    j=0;
+	    do
+	    {
+		    runState->evolve(runState);
+		    logLikelihoods[i]=runState->likelihood(runState->livePoints[i],runState->data,threadState->model);
+		    if(!sprinklewarning && j++==100) { fprintf(stderr,"Warning: having difficulty sampling prior, check your prior bounds\n"); sprinklewarning=1;}
+	    }while(isnan(logLikelihoods[i]) || isinf(logLikelihoods[i]));
+	    if(XLALPrintProgressBar((double)i/(double)Nlive)) fprintf(stderr,"\n");
     }
-    fpout=fopen(outfile,"w");
-  }
-  if(fpout==NULL) {fprintf(stderr,"Unable to open output file %s!\n",outfile); exit(1);}
-  else{
-    if(setvbuf(fpout,NULL,_IOFBF,0x100000)) /* Set buffer to 1MB so as to not thrash NFS */
-      fprintf(stderr,"Warning: Unable to set output file buffer!");
-    LALInferenceAddVariable(runState->algorithmParams,"outfile",&fpout,LALINFERENCE_void_ptr_t,LALINFERENCE_PARAM_FIXED);
   }
 
   logZarray=s->logZarray->data;
@@ -927,7 +957,7 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
     LALInferenceAddVariable(runState->livePoints[i],"logw",&logw,LALINFERENCE_REAL8_t,LALINFERENCE_PARAM_OUTPUT);
     logLtmp=logLikelihoods[i];
     logLmax=logLtmp>logLmax? logLtmp : logLmax;
-    if(isnan(logLikelihoods[i])) {
+    if(isnan(logLikelihoods[i]) || isinf(logLikelihoods[i])) {
       fprintf(stderr,"Detected logL[%i]=%lf! Sanity checking...\n",i,logLikelihoods[i]);
       if(LALInferenceSanityCheck(runState))
 	exit(1);
@@ -953,13 +983,16 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
     printAdaptiveJumpSizes(stdout, threadState);
   }
   /* Write out names of parameters */
-  FILE *lout=NULL;
-  char param_list[FILENAME_MAX];
-  sprintf(param_list,"%s_params.txt",outfile);
-  lout=fopen(param_list,"w");
+  if(!HDFOUTPUT)
+  {
+      FILE *lout=NULL;
+      char param_list[FILENAME_MAX];
+      sprintf(param_list,"%s_params.txt",outfile);
+      lout=fopen(param_list,"w");
+      LALInferenceFprintParameterHeaders(lout,runState->livePoints[0]);
+      fclose(lout);
+  }
   minpos=0;
-  LALInferenceFprintParameterHeaders(lout,runState->livePoints[0]);
-  fclose(lout);
   threadState->currentParams=currentVars;
   fprintf(stdout,"Starting nested sampling loop!\n");
   /* Iterate until termination condition is met */
@@ -971,9 +1004,9 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
 	minpos=i;
     }
     logLmin=logLikelihoods[minpos];
-    if(samplePrior) logLmin=-DBL_MAX;
+    if(samplePrior) logLmin=-INFINITY;
 
-    logZnew=incrementEvidenceSamples(runState, logLikelihoods[minpos], s);
+    logZnew=incrementEvidenceSamples(runState->GSLrandom, Nlive, logLikelihoods[minpos], s);
     //deltaZ=logZnew-logZ; - set but not used
     H=mean(Harray,Nruns);
     logZ=logZnew;
@@ -1022,7 +1055,8 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
   if(__ns_saveStateFlag!=0)
     {
       if(__ns_exitFlag) fprintf(stdout,"Saving state to %s.\n",resumefilename);
-#if 0
+#ifdef HAVE_HDF5
+
       WriteNSCheckPointH5(resumefilename,runState,s);
 #else
       WriteNSCheckPoint(resumefilename,runState,s);
@@ -1032,7 +1066,6 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
     }
    /* Have we been told to quit? */
   if(__ns_exitFlag) {
-    fclose(fpout);
     exit(0);
   }
 
@@ -1081,43 +1114,50 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
   }
   /* final corrections */
   for(i=0;i<Nlive;i++){
-    logZ=logaddexp(logZ,logLikelihoods[i]+logw);
-    for(j=0;j<Nruns;j++){
-      //logwarray[j]+=LALInferenceNSSample_logt(Nlive,runState->GSLrandom);
-      logZarray[j]=logaddexp(logZarray[j],logLikelihoods[i]+logwarray[j]-log(Nlive));
-    }
-
+    logZ=incrementEvidenceSamples(runState->GSLrandom, Nlive-i, logLikelihoods[i], s);
     if(runState->logsample) runState->logsample(runState->algorithmParams,runState->livePoints[i]);
-
   }
-
-    /* Write out the evidence */
-    fclose(fpout);
-    char bayesfile[FILENAME_MAX];
-    sprintf(bayesfile,"%s_B.txt",outfile);
-    fpout=fopen(bayesfile,"w");
-    fprintf(fpout,"%lf %lf %lf %lf\n",logZ-logZnoise,logZ,logZnoise,logLmax);
-    fclose(fpout);
+  
+    LALInferenceVariables **output_array=NULL;
+    UINT4 N_output_array=0;
+    if(LALInferenceCheckVariable(runState->algorithmParams,"outputarray")
+            &&LALInferenceCheckVariable(runState->algorithmParams,"N_outputarray") )
+    {
+            output_array=*(LALInferenceVariables ***)LALInferenceGetVariable(runState->algorithmParams,"outputarray");
+            N_output_array=*(UINT4 *)LALInferenceGetVariable(runState->algorithmParams,"N_outputarray");
+    }
     double logB=logZ-logZnoise;
     /* Pass output back through algorithmparams */
     LALInferenceAddVariable(runState->algorithmParams,"logZ",(void *)&logZ,LALINFERENCE_REAL8_t,LALINFERENCE_PARAM_OUTPUT);
     LALInferenceAddVariable(runState->algorithmParams,"logB",(void *)&logB,LALINFERENCE_REAL8_t,LALINFERENCE_PARAM_OUTPUT);
     LALInferenceAddVariable(runState->algorithmParams,"logLmax",(void *)&logLmax,LALINFERENCE_REAL8_t,LALINFERENCE_PARAM_OUTPUT);
 
-    LALInferenceVariables **output_array=NULL;
-
-    UINT4 N_output_array=0;
-    if(LALInferenceCheckVariable(runState->algorithmParams,"outputarray")
-      &&LALInferenceCheckVariable(runState->algorithmParams,"N_outputarray") )
+    /* Write out the evidence */
+    if(!HDFOUTPUT)
     {
-      output_array=*(LALInferenceVariables ***)LALInferenceGetVariable(runState->algorithmParams,"outputarray");
-      N_output_array=*(UINT4 *)LALInferenceGetVariable(runState->algorithmParams,"N_outputarray");
+        fpout = fopen(outfile,"w");
+        for(i=0;i<N_output_array;i++)
+		{
+				LALInferencePrintSample(fpout,output_array[i]);
+				fprintf(fpout,"\n");
+		}
+        fclose(fpout);
+    
+        char bayesfile[FILENAME_MAX];
+        sprintf(bayesfile,"%s_B.txt",outfile);
+        fpout=fopen(bayesfile,"w");
+        fprintf(fpout,"%lf %lf %lf %lf\n",logZ-logZnoise,logZ,logZnoise,logLmax);
+        fclose(fpout);
+    
+        
+     
     }
     /* Write HDF5 file */
-    if((ppt=LALInferenceGetProcParamVal(runState->commandLine,"--outhdf")))
+    if(HDFOUTPUT)
     {
-      LALH5File *h5file=XLALH5FileOpen(ppt->value, "w");
-      /* Create group heirarchy */
+      
+      LALH5File *h5file=XLALH5FileOpen(outfile, "w");
+      // Create group heirarchy 
       char runID[256]="";
       if((ppt=LALInferenceGetProcParamVal(runState->commandLine,"--runid")))
         snprintf(runID,255,"%s_%s","lalinference_nest",ppt->value);
@@ -1126,7 +1166,7 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
 
       LALH5File *groupPtr = LALInferenceH5CreateGroupStructure(h5file, "lalinference", runID);
       /* Create run identifier group */
-      LALInferenceH5VariablesArray2Group(groupPtr, output_array, N_output_array, LALInferenceHDF5NestedSamplesGroupName);
+      LALInferenceH5VariablesArrayToDataset(groupPtr, output_array, N_output_array, LALInferenceHDF5NestedSamplesDatasetName);
       /* TODO: Write metadata */
       XLALH5FileAddScalarAttribute(groupPtr, "log_evidence", &logZ, LAL_D_TYPE_CODE);
       XLALH5FileAddScalarAttribute(groupPtr, "log_bayes_factor", &logB, LAL_D_TYPE_CODE);
@@ -1134,6 +1174,7 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
       XLALH5FileAddScalarAttribute(groupPtr, "log_noise_evidence", &logZnoise, LAL_D_TYPE_CODE );
       XLALH5FileAddScalarAttribute(groupPtr, "log_max_likelihood", &logLmax , LAL_D_TYPE_CODE);
       XLALH5FileAddScalarAttribute(groupPtr, "number_live_points", &Nlive, LAL_U4_TYPE_CODE);
+      XLALH5FileAddScalarAttribute(groupPtr, "log_prior_volume", &logvolume, LAL_D_TYPE_CODE);
 
       XLALH5FileClose(h5file);
     }
@@ -1150,7 +1191,6 @@ void LALInferenceNestedSamplingAlgorithm(LALInferenceRunState *runState)
   if(LALInferenceGetProcParamVal(runState->commandLine,"--resume"))
   {
     if(!access(resumefilename,W_OK)) remove(resumefilename);
-    if(!access(outfilebackup,W_OK)) remove(outfilebackup);
   }
 
   /* Free memory */
@@ -1304,7 +1344,7 @@ LALInferenceVariables *LALInferenceComputeAutoCorrelation(LALInferenceRunState *
     if(LALInferenceCheckVariable(runState->algorithmParams,"verbose")) fprintf(stderr,"Caching %i samples\n",Nnew);
 
     /* Copy independent samples */
-    REAL8 oldLogL=-DBL_MAX;
+    REAL8 oldLogL=-INFINITY;
     for(i=stride,j=*Ncache;j<Nnew+*Ncache&&i<max_iterations;i+=stride,j++)
     {
       REAL8 newlogL=*(REAL8 *)LALInferenceGetVariable(variables_array[i],"logL");
@@ -1352,7 +1392,7 @@ UINT4 LALInferenceMCMCSamplePrior(LALInferenceRunState *runState)
     LALInferenceVariables proposedParams;
     memset(&proposedParams,0,sizeof(proposedParams));
     REAL8 logLmin=*(REAL8 *)LALInferenceGetVariable(runState->algorithmParams,"logLmin");
-    REAL8 thislogL=-DBL_MAX;
+    REAL8 thislogL=-INFINITY;
     UINT4 accepted=0;
 
     REAL8 logPriorOld=*(REAL8 *)LALInferenceGetVariable(threadState->currentParams,"logPrior");
@@ -1365,7 +1405,7 @@ UINT4 LALInferenceMCMCSamplePrior(LALInferenceRunState *runState)
 
     logProposalRatio = threadState->proposal(threadState,threadState->currentParams,&proposedParams);
     REAL8 logPriorNew=runState->prior(runState, &proposedParams, threadState->model);
-    if(logPriorNew==-DBL_MAX || isnan(logPriorNew) || log(gsl_rng_uniform(runState->GSLrandom)) > (logPriorNew-logPriorOld) + logProposalRatio)
+    if(isinf(logPriorNew) || isnan(logPriorNew) || log(gsl_rng_uniform(runState->GSLrandom)) > (logPriorNew-logPriorOld) + logProposalRatio)
     {
 	/* Reject - don't need to copy new params back to currentParams */
         /*LALInferenceCopyVariables(oldParams,runState->currentParams); */
@@ -1454,8 +1494,9 @@ INT4 LALInferenceNestedSamplingCachedSampler(LALInferenceRunState *runState)
     Naccept = LALInferenceNestedSamplingSloppySample(runState);
     return Naccept;
   }
-  REAL8 logL=-DBL_MAX;
+  REAL8 logL=-INFINITY;
   REAL8 logLmin=*(REAL8 *)LALInferenceGetVariable(runState->algorithmParams,"logLmin");
+
   /* Draw the last sample from the cache and reduce the size of the cache by one
    until we find one that has a high enough likelihood */
   do {
@@ -1547,8 +1588,8 @@ INT4 LALInferenceNestedSamplingSloppySample(LALInferenceRunState *runState)
         tries=0;
         mcmc_iter++;
     	sub_iter+=subchain_length;
-        if(logLmin!=-DBL_MAX) logLnew=runState->likelihood(threadState->currentParams,runState->data,threadState->model);
-        if(logLnew>logLmin || logLmin==-DBL_MAX) /* Accept */
+        if(isfinite(logLmin)) logLnew=runState->likelihood(threadState->currentParams,runState->data,threadState->model);
+        if(logLnew>logLmin || isinf(logLmin)) /* Accept */
         {
             Naccepted++;
             /* Update information to pass back out */
@@ -1606,7 +1647,7 @@ INT4 LALInferenceNestedSamplingSloppySample(LALInferenceRunState *runState)
     LALInferenceSetVariable(runState->algorithmParams,"accept_rate",&accept_rate);
     LALInferenceSetVariable(runState->algorithmParams,"sub_accept_rate",&sub_accept_rate);
     /* Adapt the sloppy fraction toward target acceptance of outer chain */
-    if(logLmin!=-DBL_MAX){
+    if(isfinite(logLmin)){
         if((REAL8)accept_rate>Target) { sloppyfraction+=5.0/(REAL8)Nmcmc;}
         else { sloppyfraction-=5.0/(REAL8)Nmcmc;}
         if(sloppyfraction>maxsloppyfraction) sloppyfraction=maxsloppyfraction;
@@ -1669,7 +1710,7 @@ void LALInferenceSetupLivePointsArray(LALInferenceRunState *runState){
 	  do{
 	    LALInferenceDrawFromPrior( runState->livePoints[i], runState->priorArgs, runState->GSLrandom );
 	    logPrior=runState->prior(runState,runState->livePoints[i],threadState->model);
-	  }while(logPrior==-DBL_MAX || isnan(logPrior));
+	  }while(isinf(logPrior) || isnan(logPrior));
 	  /* Populate log likelihood */
 	  logLs->data[i]=runState->likelihood(runState->livePoints[i],runState->data,threadState->model);
 	  LALInferenceAddVariable(runState->livePoints[i],"logL",(void *)&(logLs->data[i]),LALINFERENCE_REAL8_t,LALINFERENCE_PARAM_OUTPUT);
