@@ -74,6 +74,68 @@ const char list_extra_parameters[34][16] = {"dchi0","dchi1","dchi2","dchi3","dch
 
 const UINT4 N_extra_params = 34;
 
+
+static int InterpolateWaveform(REAL8Vector *freqs, COMPLEX16FrequencySeries *src, COMPLEX16FrequencySeries *dest);
+static int InterpolateWaveform(REAL8Vector *freqs, COMPLEX16FrequencySeries *src, COMPLEX16FrequencySeries *dest)
+{
+  REAL8 deltaF = dest->deltaF;
+  UINT4 j=ceil(freqs->data[0] / deltaF);
+  COMPLEX16 *d=dest->data->data;
+  memset(d, 0, sizeof(*(d))*j);
+  
+  /* Loop over reduced frequency set */
+  for(UINT4 i=0;i<freqs->length-1;i++)
+  {  
+    double startpsi = carg(src->data->data[i]);
+    double startamp = cabs(src->data->data[i]);
+    double endpsi = carg(src->data->data[i+1]);
+    double endamp = cabs(src->data->data[i+1]);
+
+    double startf=freqs->data[i];
+    double endf=freqs->data[i+1];
+
+    double df = endf - startf; /* Big freq step */
+    
+    /* linear interpolation setup */
+    double dpsi = (endpsi - startpsi);
+
+    /* Catch case where phase wraps around */
+    /* NOTE: If changing this check that waveforms are not corrupted
+     * at high frequencies when dpsi/df can go slightly -ve without
+     * the phase wrapping around (e.g. TF2 1.4-1.4 srate=4096)
+     */
+    if (dpsi/df<-LAL_PI ) {dpsi+=LAL_TWOPI;}
+    
+    double dpsidf = dpsi/df;
+    double dampdf = (endamp - startamp)/df;
+
+    double damp = dampdf *deltaF;
+    
+    const double dim = sin(dpsidf*deltaF);
+    const double dre = 2.0*sin(dpsidf*deltaF*0.5)*sin(dpsidf*deltaF*0.5);
+
+    /* Loop variables */
+    double newRe,newIm,f,re,im,a;
+    for(f=j*deltaF,
+	re = cos(startpsi), im = sin(startpsi),
+        a = startamp;
+        
+        f<endf;
+        
+        j++, f+=deltaF,
+        newRe = re - dre*re-dim*im,
+        newIm = im + re*dim-dre*im,
+        re=newRe, im = newIm,
+        a += damp )
+    {
+      d[j] = a * (re + I*im);
+    }
+  }
+  memset(&(d[j]), 0, sizeof(d[j])*(dest->data->length - j));
+  return 0;
+}
+
+
 void LALInferenceTemplateNullFreqdomain(LALInferenceModel *model)
 /**********************************************/
 /* returns a frequency-domain 'null' template */
@@ -290,6 +352,28 @@ void LALInferenceROQWrapperForXLALSimInspiralChooseFDWaveformSequence(LALInferen
   /* Only use GR templates */
   LALSimInspiralTestGRParam *nonGRparams = NULL;
 
+  /* Fill in the extra parameters for testing GR, if necessary */
+  for (UINT4 k=0; k<N_extra_params; k++)
+  {
+    if(LALInferenceCheckVariable(model->params,list_extra_parameters[k]))
+    {
+      XLALSimInspiralAddTestGRParam(&nonGRparams,list_extra_parameters[k],*(REAL8 *)LALInferenceGetVariable(model->params,list_extra_parameters[k]));
+    }
+  }
+  /* Fill in PPE params if they are available */
+  char PPEparam[64]="";
+  const char *PPEnames[]={"aPPE","alphaPPE","bPPE","betaPPE",NULL};
+  for(UINT4 idx=0;PPEnames[idx];idx++)
+  {
+    for(UINT4 ppeidx=0;;ppeidx++)
+    {
+      sprintf(PPEparam, "%s%d",PPEnames[idx],ppeidx);
+      if(LALInferenceCheckVariable(model->params,PPEparam))
+        XLALSimInspiralAddTestGRParam(&nonGRparams,PPEparam,LALInferenceGetREAL8Variable(model->params,PPEparam));
+      else
+        break;
+    }
+  }
 
   /* ==== Call the waveform generator ==== */
   XLAL_TRY(ret=XLALSimInspiralChooseFDWaveformSequence (&(model->roq->hptildeLinear), &(model->roq->hctildeLinear), phi0, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI,
@@ -308,31 +392,6 @@ void LALInferenceROQWrapperForXLALSimInspiralChooseFDWaveformSequence(LALInferen
         return;
 }
 
-
-
-REAL8 fLow2fStart(REAL8 fLow, INT4 ampOrder, INT4 approximant)
-/*  Compute the minimum frequency for waveform generation */
-/*  using amplitude orders above Newtonian.  The waveform */
-/*  generator turns on all orders at the orbital          */
-/*  associated with fMin, so information from higher      */
-/*  orders is not included at fLow unless fMin is         */
-/*  sufficiently low.                                     */
-{
-  if (ampOrder == -1) {
-      if (approximant == SpinTaylorT2 || approximant == SpinTaylorT4)
-          ampOrder = MAX_PRECESSING_AMP_PN_ORDER;
-      else
-          ampOrder = MAX_NONPRECESSING_AMP_PN_ORDER;
-  }
-
-    REAL8 fStart;
-    fStart = fLow * 2./(ampOrder+2);
-    return fStart;
-}
-
-
-
-
 void LALInferenceTemplateSineGaussian(LALInferenceModel *model)
 /*****************************************************/
 /* Sine-Gaussian (burst) template.                   */
@@ -631,7 +690,7 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveform(LALInferenceModel *model)
   else
     f_low = model->fLow;
 
-  f_start = fLow2fStart(f_low, amporder, approximant);
+  f_start = XLALSimInspiralfLow2fStart(f_low, amporder, approximant);
   f_max = 0.0; /* for freq domain waveforms this will stop at ISCO. Previously found using model->fHigh causes NaNs in waveform (see redmine issue #750)*/
 
   /* ==== SPINS ==== */
@@ -1031,7 +1090,6 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
     INT4 order=-1;
     INT4 amporder;
 
-    unsigned long	i;
     static int sizeWarning = 0;
     int ret=0;
     INT4 errnum=0;
@@ -1111,7 +1169,7 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
     else
         f_low = model->fLow;
 
-    f_start = fLow2fStart(f_low, amporder, approximant);
+    f_start = XLALSimInspiralfLow2fStart(f_low, amporder, approximant);
     f_max = 0.0; /* for freq domain waveforms this will stop at ISCO. Previously found using model->fHigh causes NaNs in waveform (see redmine issue #750)*/
 
     /* ==== SPINS ==== */
@@ -1192,7 +1250,28 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
 
     /* Only use GR templates */
     LALSimInspiralTestGRParam *nonGRparams = NULL;
-
+    /* Fill in the extra parameters for testing GR, if necessary */
+    for (UINT4 k=0; k<N_extra_params; k++)
+    {
+        if(LALInferenceCheckVariable(model->params,list_extra_parameters[k]))
+        {
+            XLALSimInspiralAddTestGRParam(&nonGRparams,list_extra_parameters[k],*(REAL8 *)LALInferenceGetVariable(model->params,list_extra_parameters[k]));
+        }
+    }
+    /* Fill in PPE params if they are available */
+    char PPEparam[64]="";
+    const char *PPEnames[]= {"aPPE","alphaPPE","bPPE","betaPPE",NULL};
+    for(UINT4 idx=0; PPEnames[idx]; idx++)
+    {
+        for(UINT4 ppeidx=0;; ppeidx++)
+        {
+            sprintf(PPEparam, "%s%d",PPEnames[idx],ppeidx);
+            if(LALInferenceCheckVariable(model->params,PPEparam))
+                XLALSimInspiralAddTestGRParam(&nonGRparams,PPEparam,LALInferenceGetREAL8Variable(model->params,PPEparam));
+            else
+                break;
+        }
+    }
     INT4 Nbands=-1; /* Use optimum number of bands */
     double mc_min=1.0/pow(2,0.2); /* For min 1.0-1.0 waveform */
 
@@ -1227,7 +1306,14 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
                 default:
                     /* Another error occurred that we can't handle. Propogate upward */
                     XLALSetErrno(errnum);
-                    XLAL_ERROR_VOID(errnum,"%s: Template generation failed in XLALSimInspiralChooseFDWaveformFromCache",__func__);
+                    XLAL_ERROR_VOID(errnum,"%s: Template generation failed in XLALSimInspiralChooseFDWaveformFromCache:\n\
+XLALSimInspiralChooseFDWaveformFromCache(&hptilde, &hctilde, \
+%g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, \
+model->waveFlags(%d,%d,%d,%d,numreldata),nonGRparams,%d,%d,%d,model->waveformCache)\n",__func__,
+		      phi0, m1*LAL_MSUN_SI, m2*LAL_MSUN_SI, spin1x, spin1y, spin1z, spin2x, spin2y, spin2z,
+		      f_start, f_max, f_ref, distance, inclination, lambda1, lambda2, (int) XLALSimInspiralGetSpinOrder(model->waveFlags),
+		      (int) XLALSimInspiralGetTidalOrder(model->waveFlags),(int) XLALSimInspiralGetFrameAxis(model->waveFlags),
+		      (int) XLALSimInspiralGetModesChoice(model->waveFlags),amporder, order, approximant);
             }
         }
 
@@ -1239,70 +1325,10 @@ void LALInferenceTemplateXLALSimInspiralChooseWaveformPhaseInterpolated(LALInfer
             XLALPrintError(" ERROR in LALInferenceTemplateXLALSimInspiralChooseWaveform(): encountered unallocated 'hctilde'.\n");
             XLAL_ERROR_VOID(XLAL_EFAULT);
         }
-
-        double re_p, im_p, newRe_p, newIm_p, dre_p, dim_p;
-        double re_c, im_c, newRe_c, newIm_c, dre_c, dim_c;
-
-        UINT4 j=ceil(frequencies->data[0] / model->freqhPlus->deltaF);
-
-        double f=0;
-        COMPLEX16 *modelPtrp=model->freqhPlus->data->data;
-        COMPLEX16 *modelPtrc=model->freqhCross->data->data;
-        memset(modelPtrp, 0, sizeof(*(modelPtrp))*j);
-        memset(modelPtrc, 0, sizeof(*(modelPtrc))*j);
-
-        double oldpsi_p = atan2(cimag(hptilde->data->data[0]),creal(hptilde->data->data[0]));
-        double oldpsi_c = atan2(cimag(hctilde->data->data[0]),creal(hctilde->data->data[0]));
-
-
-        for(i=0;i<hptilde->data->length-1;i++)
-        {
-            double startpsi_p = oldpsi_p;
-            double endpsi_p=atan2(cimag(hptilde->data->data[i+1]),creal(hptilde->data->data[i+1]));
-
-            double startpsi_c = oldpsi_c;
-            double endpsi_c=atan2(cimag(hctilde->data->data[i+1]),creal(hctilde->data->data[i+1]));
-
-            double startf=frequencies->data[i];
-            double endf=frequencies->data[i+1];
-            double df=endf - startf;
-
-            double dpsidf_p=(endpsi_p - startpsi_p)/df;
-            double dpsidf_c=(endpsi_c - startpsi_c)/df;
-
-            if (dpsidf_p<0.0){
-                dpsidf_p = dpsidf_p + 2*LAL_PI/df;
-            }
-            if (dpsidf_c<0.0){
-                dpsidf_c = dpsidf_c + 2*LAL_PI/df;
-            }
-
-            dim_p = sin(dpsidf_p*model->freqhPlus->deltaF);
-            dre_p = 2.0*sin(dpsidf_p*model->freqhPlus->deltaF*0.5)*sin(dpsidf_p*model->freqhPlus->deltaF*0.5);
-
-            dim_c = sin(dpsidf_c*model->freqhCross->deltaF);
-            dre_c = 2.0*sin(dpsidf_c*model->freqhCross->deltaF*0.5)*sin(dpsidf_c*model->freqhCross->deltaF*0.5);
-
-            for(f=j*model->freqhPlus->deltaF, re_p = 1.0, im_p = 0.0, re_c = 1.0, im_c = 0.0;
-                f<endf;
-                j++, f+=model->freqhPlus->deltaF,
-                newRe_p=re_p - dre_p*re_p-dim_p*im_p,
-                newIm_p = im_p + re_p*dim_p-dre_p*im_p,
-                re_p=newRe_p, im_p = newIm_p,
-                newRe_c=re_c - dre_c*re_c-dim_c*im_c,
-                newIm_c = im_c + re_c*dim_c-dre_c*im_c,
-                re_c=newRe_c, im_c = newIm_c)
-            {
-                modelPtrp[j]=hptilde->data->data[i]*(re_p +I*im_p);
-                modelPtrc[j]=hctilde->data->data[i]*(re_c +I*im_c);
-            }
-            oldpsi_p = endpsi_p;
-            oldpsi_c = endpsi_c;
-            //  if(f>fmax_checked) break;
-        }
-        memset(&(modelPtrp[j]), 0, sizeof(COMPLEX16)*(model->freqhPlus->data->length - j));
-        memset(&(modelPtrc[j]), 0, sizeof(COMPLEX16)*(model->freqhCross->data->length - j));
-
+        
+        
+        InterpolateWaveform(frequencies, hptilde, model->freqhPlus);
+        InterpolateWaveform(frequencies, hctilde, model->freqhCross);
 
         /* Destroy the nonGr params */
         XLALSimInspiralDestroyTestGRParam(nonGRparams);
